@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: client.c,v 1.71 2008/03/27 20:57:19 evertonm Exp $ */
+/* $Id: client.c,v 1.72 2008/07/10 21:59:26 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -938,7 +938,8 @@ static void spawn_one_tcp_client(const char *hostport,
 				     nodelay,
 				     nepim_global.tos,
 				     nepim_global.socket_keepalive,
-				     nepim_global.client_tcp_mss);
+				     nepim_global.client_tcp_mss,
+				     0 /* join iface */);
     if (sd < 0) {
       if (ai->ai_family == AF_UNIX)
 	fprintf(stderr, 
