@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: common.c,v 1.93 2007/10/19 20:10:18 evertonm Exp $ */
+/* $Id: common.c,v 1.95 2007/11/28 15:47:19 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +39,7 @@ MA 02110-1301 USA
 #include "rand.h"
 
 
-const char * const NEPIM_LABEL_PARTIAL      = "prt";
+const char * const NEPIM_LABEL_PARTIAL      = "cur";
 const char * const NEPIM_LABEL_TOTAL        = "avg";
 const char * const NEPIM_LABEL_TOTAL_BROKEN = "avg(broken)";
 const char * const NEPIM_LABEL_MIN          = "min";
@@ -51,11 +51,6 @@ const char * const NEPIM_MSG_KEEPALIVES_MISS  = "peer not sending keepalives? (s
 static int line = 0;
 
 
-const char *nepim_version()
-{
-  return "0.44";
-}
-
 static void header(FILE *out, int ident, int udp)
 {
   if (!(line++ % 10)) {
