@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.73 2008/05/20 13:17:29 evertonm Exp $
+  $Id: main.c,v 1.74 2008/07/11 15:26:33 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -170,6 +170,10 @@ static void usage(FILE *out, const char *prog_name)
           "                 to join an SSM (source,group) pair, use: -j source+group\n"
 	  "                 examples: -j fe80::1+ff01::1111\n"
 	  "                           -j 10.10.10.10+232.1.1.1,2000\n"
+	  "                 append @interface to specify an interface\n"
+	  "                 examples: -j 239.1.1.1@eth0             (by name)\n"
+	  "                           -j 1.1.1.1+232.1.1.1@10.0.0.1 (by address)\n"
+	  "                           -j 239.1.1.1@1                (by index)\n"
 	  "  -U path-list   bind UNIX stream sockets to specified addresses\n"
 #if 0
 	  "  -G path-list   bind UNIX datagram sockets to specified addresses\n"
