@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: server.c,v 1.84 2008/05/30 16:19:07 evertonm Exp $ */
+/* $Id: server.c,v 1.86 2008/07/11 15:26:33 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1032,7 +1032,7 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 				      nodelay,
 				      nepim_global.tos,
 				      nepim_global.server_tcp_mss,
-				      -1, /* iface_index */
+				      0,  /* join_iface */
 				      0,  /* ssm_src_addr */
 				      -1  /* ssm_src_addrlen */);
     if (sd < 0) {
@@ -1095,7 +1095,7 @@ static int spawn_unix_listener(const char *path)
 				    nodelay,
 				    nepim_global.tos,
 				    nepim_global.server_tcp_mss,
-				    -1, /* iface_index */
+				    0, /* iface_index */
 				    0,  /* ssm_src_addr */
 				    -1  /* ssm_src_addrlen */);
   if (sd < 0) {
@@ -1120,7 +1120,8 @@ static int spawn_unix_listener(const char *path)
 extern int nepim_udp_listener(const char *ssm_source,
 			      const char *hostname,
                               const char *portname,
-			      int join);
+			      int join,
+			      const char *join_iface);
 
 static void parse_listeners(const char *list, int *udp, int *tcp, int join)
 {
@@ -1150,6 +1151,7 @@ static void parse_listeners(const char *list, int *udp, int *tcp, int join)
     if (join) {
       char *group;
       char *ssm_source;
+      char *iface;
 
       if (addr_split_sourcegroup(buf, sizeof(buf), &group)) {
 	ssm_source = 0;
@@ -1158,14 +1160,20 @@ static void parse_listeners(const char *list, int *udp, int *tcp, int join)
       else {
 	ssm_source = buf;
       }
+
+      /* split group into group@iface */
+      if (addr_split_iface(group, buf + sizeof(buf) - group, &iface)) {
+	iface = 0;
+      }
+
       *tcp += spawn_tcp_listener(group, portname);
-      *udp += nepim_udp_listener(ssm_source, group, portname, 1 /* join */);
+      *udp += nepim_udp_listener(ssm_source, group, portname, 1 /* join */, iface);
 
       continue; /* for */
     }
 
     *tcp += spawn_tcp_listener(buf, portname);
-    *udp += nepim_udp_listener(0, buf, portname, 0 /* join */);
+    *udp += nepim_udp_listener(0, buf, portname, 0 /* join */, 0 /* iface */);
 
   } /* for */
 }
@@ -1313,10 +1321,12 @@ void nepim_server_run()
     if (!nepim_global.no_inet6)
       udp_listeners += nepim_udp_listener(0 /* ssm_source */,
 					  INET6_ANY, nepim_global.portname,
-                                          0 /* no join */);
+                                          0 /* no join */,
+					  0 /* join_iface */);
     udp_listeners += nepim_udp_listener(0 /* ssm_source */,
 					INET_ANY, nepim_global.portname,
-                                        0 /* no join */);
+                                        0 /* no join */,
+					0 /* join_iface */);
   }
 
 #if 0
