@@ -18,11 +18,12 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: session.c,v 1.17 2007/08/17 17:12:48 evertonm Exp $ */
+/* $Id: session.c,v 1.18 2007/10/18 21:51:44 evertonm Exp $ */
 
 #include <string.h>
 #include <assert.h>
 #include <time.h>
+#include <float.h>
 
 #include "session.h"
 #include "common.h"
@@ -70,6 +71,7 @@ void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
   session->sweep_step              = opt->sweep_step;
   session->write_floor             = opt->write_floor;
   session->write_ceil              = opt->write_ceil;
+  session->report_partial_min_max  = opt->report_partial_min_max;
 
   session->duration_done        = 0;
   session->total_reads          = 0;
@@ -84,6 +86,20 @@ void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
   session->write_sweep_rand_ctx = time(0); /* random seed (see rand.c) */
   session->udp_dst_random_ctx   = session->write_sweep_rand_ctx;
   session->write_sweep_current  = session->write_floor;
+
+  session->min.kbps_in   = FLT_MAX;
+  session->min.kbps_out  = FLT_MAX;
+  session->min.pps_in    = FLT_MAX;
+  session->min.pps_out   = FLT_MAX;
+  session->min.pkt_loss  = FLT_MAX;
+  session->min.pkt_ooo   = FLT_MAX;
+
+  session->max.kbps_in   = FLT_MIN;
+  session->max.kbps_out  = FLT_MIN;
+  session->max.pps_in    = FLT_MIN;
+  session->max.pps_out   = FLT_MIN;
+  session->max.pkt_loss  = FLT_MIN;
+  session->max.pkt_ooo   = FLT_MIN;
 }
 
 void nepim_session_write_add(nepim_session_t *session, int len)
