@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: sock.c,v 1.47 2008/05/27 13:32:25 evertonm Exp $ */
+/* $Id: sock.c,v 1.53 2008/07/11 21:40:01 evertonm Exp $ */
 
 #include <assert.h>
 #include <netdb.h>
@@ -34,8 +34,13 @@ MA 02110-1301 USA
 #include <arpa/inet.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <net/if.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <net/if.h>
 
 #include "sock.h"
+#include "conf.h"
 
 #ifndef SOL_IP
 #define SOL_IP 0
@@ -110,6 +115,7 @@ struct group_source_req {
 #define NEPIM_SOCK_ERR_TOS                    (-18)
 #define NEPIM_SOCK_ERR_SOCK_KA                (-19)
 #define NEPIM_SOCK_ERR_MSS                    (-20)
+#define NEPIM_SOCK_ERR_MCAST_JOIN_IFACE       (-21)
 
 int nepim_sock_get_port(const struct sockaddr *addr)
 {
@@ -159,6 +165,17 @@ int nepim_sock_family(const struct sockaddr *addr)
   return sa->inet.sin_family;
 }
 
+int nepim_sock_addrlen(const struct sockaddr *addr)
+{
+  switch (nepim_sock_family(addr)) {
+  case PF_INET:  return sizeof(struct sockaddr_in);
+  case PF_INET6: return sizeof(struct sockaddr_in6);
+  default:
+    assert(0);
+  }
+  assert(0);
+}
+
 void nepim_sock_dump_addr(char *buf, int buf_size,
 			  const struct sockaddr *addr,
 			  int addr_len)
@@ -410,13 +427,184 @@ static int create_socket(FILE *err, int domain, int type, int protocol,
   return sd;
 }
 
+static int iface_solve_index(const char *ifname)
+{
+  struct if_nameindex *ini;
+  struct if_nameindex *i;
+  int ifindex = -1;
+
+  assert(ifname);
+
+  ini = if_nameindex();
+  if (!ini) {
+    if (nepim_global.verbose_stderr) {
+      int err = errno;
+      fprintf(nepim_global.verbose_stderr,
+	      "%s: %s: interface=%s: failure solving index: errno=%d: %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      ifname, err, strerror(err));
+      errno = err;
+    }
+    return -1;
+  }
+
+  for (i = ini; i->if_index > 0; ++i) {
+    if (nepim_global.verbose_stderr) {
+      fprintf(nepim_global.verbose_stderr,
+	      "%s: %s: interface=%s matching against local ifname=%s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      ifname, i->if_name);
+    }
+    if (!strcmp(i->if_name, ifname)) {
+      ifindex = i->if_index;
+      break;
+    }
+    ++i;
+  }
+
+  if_freenameindex(ini);
+
+  return ifindex;
+}
+
+static int iface_solve_addr4(struct in_addr *ifaddr,
+			     const char *ifhost)
+{
+  struct addrinfo hints;
+  struct addrinfo *ai_res;
+  struct addrinfo *ai;
+  int result;
+
+  assert(ifaddr);
+
+  memset(&hints, 0, sizeof(hints));
+
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_protocol = 0;
+  hints.ai_flags = AI_CANONNAME;
+  hints.ai_family = PF_INET;
+  hints.ai_addrlen = 0;
+  hints.ai_addr = 0;
+  hints.ai_canonname = 0;
+
+  result = getaddrinfo(ifhost, 0, &hints, &ai_res);
+  if (result)
+    return -1;
+
+  for (ai = ai_res; ai; ai = ai->ai_next) {
+    if (ai->ai_family != PF_INET)
+      continue;
+
+    memcpy(ifaddr, &((struct sockaddr_in *) ai->ai_addr)->sin_addr, sizeof(*ifaddr));
+    break;
+  }
+
+  freeaddrinfo(ai_res);
+  return !ai;
+}
+
+static int iface_solve(int *ifindex,
+		       struct in_addr *ifaddr,
+		       const char *iface)
+{
+  int result;
+  long value;
+  const char *i;
+
+  assert(ifindex);
+  assert(ifaddr);
+  assert(iface);
+
+  /* option 1: interface literal index */
+  for (i = iface;;++i) {
+    int c = *i;
+    /* eos? */
+    if (!c) {
+      errno = 0;
+      value = strtol(iface, 0, 10);
+      if (!errno) {
+	*ifindex = value;
+	ifaddr->s_addr = htons(INADDR_ANY);
+	return 0;
+      }
+      break;
+    }
+    /* bad number? */
+    if (!isdigit(c)) {
+      errno = EINVAL;
+      if (nepim_global.verbose_stderr)
+	fprintf(nepim_global.verbose_stderr,
+		"%s: %s: interface=%s: not a literal interface index: errno=%d: %s\n",
+		__FILE__, __PRETTY_FUNCTION__, iface, errno, strerror(errno));
+      break;
+    }
+  }
+  
+  /* option 2: interface name to index */
+  result = iface_solve_index(iface);
+  if (result >= 0) {
+    *ifindex = result;
+    ifaddr->s_addr = htons(INADDR_ANY);
+    return 0;
+  }
+  if (nepim_global.verbose_stderr)
+    fprintf(nepim_global.verbose_stderr,
+	    "%s: %s: interface=%s: not an interface name\n",
+	    __FILE__, __PRETTY_FUNCTION__, iface);
+
+  /* option 3: interface address */
+  /*
+    only for PF_INET, since PF_INET6 relies on ifindex
+  */
+  result = iface_solve_addr4(ifaddr, iface);
+  if (!result) {
+    *ifindex = 0;
+    return 0;
+  }
+
+  if (nepim_global.verbose_stderr)
+    fprintf(nepim_global.verbose_stderr,
+	    "%s: %s: interface=%s: not an interface address\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    iface);
+
+  *ifindex = -1;
+  ifaddr->s_addr = htons(INADDR_ANY);
+  
+  return -1;
+}
+
 static int socket_mcast_join(int sd, int family,
-			     const struct sockaddr *addr)
+			     const struct sockaddr *addr,
+			     const char *iface)
 {
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
   } *sa = (void *) addr;
+  struct in_addr ifaddr;
+  int ifindex;
+  int result;
+
+  if (iface) { 
+    ifindex = -1;
+    if (iface_solve(&ifindex, &ifaddr, iface)) {
+
+      if (nepim_global.verbose_stderr)
+	fprintf(nepim_global.verbose_stderr,
+		"%s %s: failure solving interface=%s: ifaddr=%s ifindex=%d\n",
+		__FILE__, __PRETTY_FUNCTION__,
+		iface ? iface : "any",
+		inet_ntoa(ifaddr), ifindex);
+
+      errno = ENODEV;
+      return NEPIM_SOCK_ERR_MCAST_JOIN_IFACE;
+    }
+  }
+  else {
+    ifindex = 0;
+    ifaddr.s_addr = htons(INADDR_ANY);
+  }
 
   switch (family) {
   case PF_INET:
@@ -424,14 +612,14 @@ static int socket_mcast_join(int sd, int family,
 #ifdef HAVE_IP_MREQN
       struct ip_mreqn opt;
       opt.imr_multiaddr = sa->inet.sin_addr;
-      opt.imr_address.s_addr = htons(INADDR_ANY);
-      opt.imr_ifindex = 0;
-      return setsockopt(sd, SOL_IP, IP_ADD_MEMBERSHIP, &opt, sizeof(opt));
+      opt.imr_ifindex = ifindex;
+      opt.imr_address = ifaddr;
+      result = setsockopt(sd, SOL_IP, IP_ADD_MEMBERSHIP, &opt, sizeof(opt));
 #else
       struct ip_mreq opt;
       opt.imr_multiaddr = sa->inet.sin_addr;
-      opt.imr_interface.s_addr = htons(INADDR_ANY);
-      return setsockopt(sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &opt, sizeof(opt));
+      opt.imr_interface = ifaddr;
+      result = setsockopt(sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &opt, sizeof(opt));
 #endif
     }
     break;
@@ -439,44 +627,108 @@ static int socket_mcast_join(int sd, int family,
     {
       struct ipv6_mreq opt;
 
+      opt.ipv6mr_interface = ifindex;
       assert(sizeof(opt.ipv6mr_multiaddr.s6_addr) == 
 	     sizeof(sa->inet6.sin6_addr.s6_addr));
       memcpy(&opt.ipv6mr_multiaddr.s6_addr, 
 	     &sa->inet6.sin6_addr.s6_addr, 
 	     sizeof(opt.ipv6mr_multiaddr.s6_addr));
-      opt.ipv6mr_interface = 0;
 
-      return setsockopt(sd, SOL_IPV6, IPV6_ADD_MEMBERSHIP, &opt, sizeof(opt));
+      result = setsockopt(sd, SOL_IPV6, IPV6_ADD_MEMBERSHIP, &opt, sizeof(opt));
     }
     break;
   default:
     assert(0);
   }
 
-  assert(0);
+  if (result) {
+    int err = errno;
 
-  return NEPIM_SOCK_ERR_MCAST_JOIN;
+    if (nepim_global.verbose_stderr) {
+      char buf_grp[300];
+
+      nepim_sock_dump_addr(buf_grp, sizeof(buf_grp), addr, nepim_sock_addrlen(addr));
+
+      fprintf(nepim_global.verbose_stderr,
+	      "%s %s: failure joining %s@%s ifaddr=%s ifindex=%d: errno=%d: %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      buf_grp, iface ? iface : "any",
+	      inet_ntoa(ifaddr), ifindex,
+	      err, strerror(err));
+    }
+  
+    errno = err;
+  }
+
+  return result;
 }
 
-static int socket_mcast_join_sourcegroup(int sd, int family, int iface_index,
+static int socket_mcast_join_sourcegroup(int sd, int family,
 					 const struct sockaddr *src,
-					 const struct sockaddr *grp)
+					 const struct sockaddr *grp,
+					 const char *iface)
 {
   struct group_source_req group_source_req;
+  struct in_addr ifaddr;
+  int result;
+  int ifindex;
 
-  group_source_req.gsr_interface = iface_index;
+  if (iface) { 
+    ifindex = -1;
+    if (iface_solve(&ifindex, &ifaddr, iface)) {
+
+      if (nepim_global.verbose_stderr)
+	fprintf(nepim_global.verbose_stderr,
+		"%s %s: failure solving interface=%s: ifaddr=%s ifindex=%d\n",
+		__FILE__, __PRETTY_FUNCTION__,
+		iface ? iface : "any",
+		inet_ntoa(ifaddr), ifindex);
+      
+      errno = ENODEV;
+      return NEPIM_SOCK_ERR_MCAST_JOIN_IFACE;
+    }
+
+    /* got null ifindex while user didn't request it explicitely? */
+    if (!ifindex && strcmp(iface, "0")) {
+
+      if (nepim_global.verbose_stderr)
+	fprintf(nepim_global.verbose_stderr,
+		"%s %s: could not find non-null ifindex for interface=%s: ifaddr=%s ifindex=%d\n",
+		__FILE__, __PRETTY_FUNCTION__,
+		iface ? iface : "any",
+		inet_ntoa(ifaddr), ifindex);
+	
+      errno = ENOTSUP;
+      return NEPIM_SOCK_ERR_MCAST_JOIN_IFACE;
+    }
+  }
+  else {
+    ifindex = 0;
+  }
+
+  if (ifindex < 0) {
+    fprintf(stderr,
+	    "%s: %s: interface=%s ifaddr=%s index=%d: could not find interface index (FIXME: ifaddr => index)\n",
+	    __FILE__, __PRETTY_FUNCTION__, iface, inet_ntoa(ifaddr), ifindex);
+    errno = ENODEV;
+    return NEPIM_SOCK_ERR_MCAST_JOIN_IFACE;
+  }
+
+  group_source_req.gsr_interface = ifindex;
 
   switch (family) {
   case PF_INET:
     {
+
       assert(sizeof(group_source_req.gsr_source) >= sizeof(struct sockaddr_in));
       assert(sizeof(group_source_req.gsr_group) >= sizeof(struct sockaddr_in));
 
       memcpy(&group_source_req.gsr_source, src, sizeof(struct sockaddr_in));
       memcpy(&group_source_req.gsr_group, grp, sizeof(struct sockaddr_in));
 
-      return setsockopt(sd, SOL_IP, MCAST_JOIN_SOURCE_GROUP,
-			&group_source_req, sizeof(group_source_req));
+      result = setsockopt(sd, SOL_IP, MCAST_JOIN_SOURCE_GROUP,
+			  &group_source_req, sizeof(group_source_req));
+      break;
     }
   case PF_INET6:
     {
@@ -486,16 +738,38 @@ static int socket_mcast_join_sourcegroup(int sd, int family, int iface_index,
       memcpy(&group_source_req.gsr_source, src, sizeof(struct sockaddr_in6));
       memcpy(&group_source_req.gsr_group, grp, sizeof(struct sockaddr_in6));
 
-      return setsockopt(sd, SOL_IPV6, MCAST_JOIN_SOURCE_GROUP,
-			&group_source_req, sizeof(group_source_req));
+      result = setsockopt(sd, SOL_IPV6, MCAST_JOIN_SOURCE_GROUP,
+			  &group_source_req, sizeof(group_source_req));
+      break;
     }
   default:
     assert(0);
   }
 
-  assert(0);
+  if (result) {
+    int err = errno;
 
-  return NEPIM_SOCK_ERR_MCAST_JOIN_SOURCEGROUP;
+    if (nepim_global.verbose_stderr) {
+      char buf_src[300];
+      char buf_grp[300];
+
+      nepim_sock_dump_addr(buf_src, sizeof(buf_src), src, nepim_sock_addrlen(src));
+      nepim_sock_dump_addr(buf_grp, sizeof(buf_grp), grp, nepim_sock_addrlen(grp));
+
+      fprintf(nepim_global.verbose_stderr,
+	      "%s %s: failure joining %s+%s@%s ifaddr=%s ifindex=%d: errno=%d: %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      buf_src, buf_grp, iface ? iface : "any",
+	      inet_ntoa(ifaddr),
+	      group_source_req.gsr_interface,
+	      err,
+	      strerror(err));
+    }
+  
+    errno = err;
+  }
+
+  return result;
 }
 
 int nepim_create_socket(FILE *err,
@@ -511,7 +785,8 @@ int nepim_create_socket(FILE *err,
 			int nodelay,
 			int tos,
 			int mss,
-			int iface, const struct sockaddr *ssm_src_addr, int ssm_src_addrlen)
+			const char *iface,
+			const struct sockaddr *ssm_src_addr, int ssm_src_addrlen)
 {
   int sd;
   int result;
@@ -522,15 +797,16 @@ int nepim_create_socket(FILE *err,
     return sd;
 
 #if 0
-      {
-	  char buf1[500];
-	  char buf2[500];
-	  nepim_sock_dump_addr(buf1, sizeof(buf1), ssm_src_addr, ssm_src_addrlen);
-	  nepim_sock_dump_addr(buf2, sizeof(buf2), addr, addr_len);
-	  fprintf(stderr, "DEBUG %s %s: mcast_join=%d mcast_join_sourcegroup(%d,%s,%s,%d)\n",
-		  __FILE__, __PRETTY_FUNCTION__,
-		  mcast_join, sd, buf1, buf2, nepim_sock_get_port(addr));
-      }
+  {
+    char buf1[500];
+    char buf2[500];
+    nepim_sock_dump_addr(buf1, sizeof(buf1), ssm_src_addr, ssm_src_addrlen);
+    nepim_sock_dump_addr(buf2, sizeof(buf2), addr, addr_len);
+    fprintf(stderr, "DEBUG %s %s: mcast_join=%d mcast_join_sourcegroup(%d,%s,%s,%d,%s)\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    mcast_join, sd, buf1, buf2, nepim_sock_get_port(addr),
+	    iface ? iface : "any");
+  }
 #endif
 
   if (mcast_join) {
@@ -543,7 +819,7 @@ int nepim_create_socket(FILE *err,
     /* SSM join (S,G) ? */
     if (ssm_src_addr) {
 
-      if (socket_mcast_join_sourcegroup(sd, family, iface, ssm_src_addr, addr)) {
+      if (socket_mcast_join_sourcegroup(sd, family, ssm_src_addr, addr, iface)) {
 	
 	if (err) {
 	  char buf1[500];
@@ -553,9 +829,10 @@ int nepim_create_socket(FILE *err,
 	  nepim_sock_dump_addr(buf1, sizeof(buf1), ssm_src_addr, ssm_src_addrlen);
 	  nepim_sock_dump_addr(buf2, sizeof(buf2), addr, addr_len);
 	  
-	  fprintf(err, "%s: %s: mcast_join_sourcegroup(%d,%s,%s,%d): errno=%d: %s\n",
+	  fprintf(err, "%s: %s: mcast_join_sourcegroup(%d,%s,%s,%d,%s): errno=%d: %s\n",
 		  __FILE__, __PRETTY_FUNCTION__,
 		  sd, buf1, buf2, nepim_sock_get_port(addr),
+		  iface ? iface : "any",
 		  e, strerror(e));
 	}
 	
@@ -565,7 +842,7 @@ int nepim_create_socket(FILE *err,
     } /* ssm_src_addr */
     
       /* Regular mcast join (*,G) */
-    else if (socket_mcast_join(sd, family, addr)) {
+    else if (socket_mcast_join(sd, family, addr, iface)) {
 
       if (err) {
 	char buf[500];
@@ -573,9 +850,10 @@ int nepim_create_socket(FILE *err,
 
 	nepim_sock_dump_addr(buf, sizeof(buf), addr, addr_len);
 
-	fprintf(err, "%s: %s: mcast_join(%d,%s,%d): errno=%d: %s\n",
+	fprintf(err, "%s: %s: mcast_join(%d,%s,%d,%s): errno=%d: %s\n",
 		__FILE__, __PRETTY_FUNCTION__,
 		sd, buf, nepim_sock_get_port(addr),
+		iface ? iface : "any",
 		e, strerror(e));
       }
 
@@ -634,7 +912,8 @@ int nepim_create_listener_socket(FILE *err,
 				 int nodelay,
 				 int tos,
 				 int mss,
-				 int iface, const struct sockaddr *ssm_src_addr, int ssm_src_addrlen)
+				 const char *iface,
+				 const struct sockaddr *ssm_src_addr, int ssm_src_addrlen)
 {
   int sd;
 
@@ -844,7 +1123,8 @@ int nepim_connect_client_socket(FILE *err,
 				int nodelay,
 				int tos,
 				int socket_keepalive,
-				int mss)
+				int mss,
+				const char *iface)
 {
   int sd;
   int result;
@@ -853,7 +1133,7 @@ int nepim_connect_client_socket(FILE *err,
 			   family, type, protocol, pmtu_mode, ttl,
 			   0 /* mcast_join: false */,
 			   win_recv, win_send, nodelay, tos, mss,
-			   -1, /* iface */
+			   iface,
 			   0,  /* ssm_src_addr */
 			   -1  /* ssm_src_addrlen */);
   if (sd < 0)
@@ -867,7 +1147,7 @@ int nepim_connect_client_socket(FILE *err,
     int one = 1;
     if (setsockopt(sd, SOL_SOCKET, SO_BSDCOMPAT, &one, sizeof(one)))
       return -1;
- }
+  }
 #endif /* Linux SO_BSDCOMPAT */
 
   result = nepim_socket_keepalive(err, sd, socket_keepalive);
