@@ -51,7 +51,8 @@ int nepim_create_socket(FILE *err,
 			int nodelay,
 			int tos,
 			int mss,
-			int iface, const struct sockaddr *ssm_src_addr, int ssm_src_addrlen);
+			const char *iface,
+			const struct sockaddr *ssm_src_addr, int ssm_src_addrlen);
 int nepim_create_listener_socket(FILE *err,
 				 struct sockaddr *addr,
 				 int addr_len,
@@ -66,7 +67,8 @@ int nepim_create_listener_socket(FILE *err,
 				 int nodelay,
 				 int tos,
 				 int mss,
-				 int iface, const struct sockaddr *ssm_src_addr, int ssm_src_addrlen);
+				 const char *iface,
+				 const struct sockaddr *ssm_src_addr, int ssm_src_addrlen);
 int nepim_connect_client_socket(FILE *err,
 				const struct sockaddr *bind, int bind_len,
 				struct sockaddr *addr, int addr_len,
@@ -80,14 +82,15 @@ int nepim_connect_client_socket(FILE *err,
 				int nodelay,
 				int tos,
 				int socket_keepalive,
-				int mss);
-
+				int mss,
+				const char *iface);
 int nepim_sock_get_port(const struct sockaddr *addr);
 void nepim_sock_set_port(const struct sockaddr *addr, int port);
 void nepim_sock_dump_addr(char *buf, int buf_size,
 			  const struct sockaddr *addr,
 			  int addr_len);
 int nepim_sock_family(const struct sockaddr *addr);
+int nepim_sock_addrlen(const struct sockaddr *addr);
 void nepim_sock_show_opt(FILE *err, FILE *out, int sd);
 
 #endif /* NEPIM_SOCK_H */
