@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: str.c,v 1.9 2008/05/20 11:20:49 evertonm Exp $ */
+/* $Id: str.c,v 1.10 2008/07/11 15:26:33 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -169,6 +169,11 @@ int nepim_str_split_plus(char *buf, int buf_size, char **next)
   return nepim_str_split('+', buf, buf_size, next);
 }
 
+int nepim_str_split_at(char *buf, int buf_size, char **next)
+{
+  return nepim_str_split('@', buf, buf_size, next);
+}
+
 int addr_split_port(char *buf, int buf_size, char **port)
 {
   return nepim_str_split_comma(buf, buf_size, port);
@@ -179,6 +184,11 @@ int addr_split_sourcegroup(char *buf, int buf_size, char **group)
   return nepim_str_split_plus(buf, buf_size, group);
 }
 
+int addr_split_iface(char *buf, int buf_size, char **iface)
+{
+  return nepim_str_split_at(buf, buf_size, iface);
+}
+
 static int multiplier_affix(const char *str)
 {
   const char *i;
