@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_client.c,v 1.85 2008/03/27 20:57:20 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.86 2008/07/10 21:59:26 evertonm Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -1308,7 +1308,8 @@ static int spawn_udp_connection(const char *hostport,
 				     0 /* tcp nodelay */,
 				     nepim_global.tos,
 				     nepim_global.socket_keepalive,
-				     nepim_global.client_tcp_mss);
+				     nepim_global.client_tcp_mss,
+				     0 /* join iface */);
     if (sd < 0) {
       if (ai->ai_family == AF_UNIX)
 	fprintf(stderr, 
